// Etape 1 :
// Importation des packages Java 2
import java.applet.Applet;
import java.awt.*;

// Etape 2 :
// Importation des packages Java 3D
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class Collision extends Applet {

  public Collision() {
    this.setLayout(new BorderLayout());

    // Etape 3 :
    // Creation du Canvas 3D
    Canvas3D canvas3D =
        new Canvas3D(SimpleUniverse.getPreferredConfiguration());

    this.add(canvas3D, BorderLayout.CENTER);

    // Etape 4 :
    // Creation d'un objet SimpleUniverse
    SimpleUniverse simpleU = new SimpleUniverse(canvas3D);

    // Etape 5 :
    // Positionnement du point d'observation pour avoir une vue correcte de la
    // scene 3D
    simpleU.getViewingPlatform().setNominalViewingTransform();

    // Etape 6 :
    // Creation de la scene 3D qui contient tous les objets 3D que l'on veut
    // visualiser
    BranchGroup scene = createSceneGraph(canvas3D);

    // Etape 7 :
    // Compilation de la scene 3D
    scene.compile();

    // Etape 8:
    // Attachement de la scene 3D a l'objet SimpleUniverse
    simpleU.addBranchGraph(scene);
  }

  /**
   * Creation de la scene 3D qui contient tous les objets 3D
   * @param canvas canvas 3D
   * @return scene 3D
   */
  public BranchGroup createSceneGraph(Canvas3D canvas) {
    // Creation de l'objet parent qui contiendra tous les autres objets 3D
    BranchGroup parent = new BranchGroup();

    /**********************************************************************
     * Creation des 2 spheres statiques (non selectionnables a la souris) *
     **********************************************************************/
    Sphere sphereGauche = new Sphere(0.3f);
    sphereGauche.setUserData("Sphere gauche");
    Sphere sphereDroite = new Sphere(0.3f);
    sphereDroite.setUserData("Sphere droite");

    // Creation du materiau qui possede une couleur d'emission blanche
    // pour les 2 spheres
    Material emissiveColor = new Material();
    emissiveColor.setEmissiveColor(new Color3f(Color.white));

    // Creation de l'apparence qui contient le materiau precedemment cree
    Appearance apparenceGauche = new Appearance();
    apparenceGauche.setMaterial(emissiveColor);
    sphereGauche.setAppearance(apparenceGauche);
    Appearance apparenceDroite = new Appearance();
    apparenceDroite.setMaterial(emissiveColor);
    sphereDroite.setAppearance(apparenceDroite);

    // Ajout des capacites pour modifier la couleur (materiau) des spheres
    apparenceGauche.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
    apparenceDroite.setCapability(Appearance.ALLOW_MATERIAL_WRITE);

    // Positionnement de la sphere gauche dans la scene 3D
    Transform3D sphereTranslationGauche = new Transform3D();
    sphereTranslationGauche.setTranslation(new Vector3f(-0.6f, 0.6f, 0f));
    TransformGroup sphereTranslationGaucheTG =
        new TransformGroup(sphereTranslationGauche);

    // Positionnement de la sphere droite dans la scene 3D
    Transform3D sphereTranslationDroite = new Transform3D();
    sphereTranslationDroite.setTranslation(new Vector3f(0.6f, 0.6f, 0f));
    TransformGroup sphereTranslationDroiteTG =
        new TransformGroup(sphereTranslationDroite);

    // Ajout de la sphere gauche dans le graphe
    sphereTranslationGaucheTG.addChild(sphereGauche);
    parent.addChild(sphereTranslationGaucheTG);

    // Ajout de la sphere droite dans le graphe
    sphereTranslationDroiteTG.addChild(sphereDroite);
    parent.addChild(sphereTranslationDroiteTG);

    // Creation d'un comportement "collision" pour la sphere gauche
    DetecteurCollision detectCollisionGauche =
        new DetecteurCollision(sphereGauche,
                               apparenceGauche,
                               new BoundingSphere());
    parent.addChild(detectCollisionGauche);

    // Creation d'un comportement "collision" pour la sphere droite
    DetecteurCollision detectCollisionDroite =
        new DetecteurCollision(sphereDroite,
                               apparenceDroite,
                               new BoundingSphere());
    parent.addChild(detectCollisionDroite);
    /**********************************************************************
     Fin creation des 2 spheres statiques (non selectionnables a la souris)
     **********************************************************************/

    /******************************************************
     * Creation du cube mobile selectionnable a la souris *
     ******************************************************/
    // Creation d'un BranchGroup contenant le cube
    BranchGroup cubeBG = new BranchGroup();

    // Creation du groupe de transformation sur lequel va s'appliquer le
    // comportement de deplacement une fois le cube selectionne
    TransformGroup pickTransform = new TransformGroup();
    pickTransform.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    pickTransform.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    pickTransform.setCapability(TransformGroup.ENABLE_PICK_REPORTING);

    // Creation et positionnement du cube mobile dans le graphe de la scene
    Transform3D translation = new Transform3D();
    translation.setTranslation(new Vector3f(0f, -0.6f, 0f));
    TransformGroup translationTG = new TransformGroup(translation);
    pickTransform.addChild(new ColorCube(0.1));
    translationTG.addChild(pickTransform);
    cubeBG.addChild(translationTG);

    // Creation du comportement de translation (au bouton gauche) une fois
    // le cube selectionne
    CustomPickTranslateBehavior pickTranslate =
        new CustomPickTranslateBehavior(cubeBG,
                                        canvas,
                                        new BoundingSphere());
    cubeBG.addChild(pickTranslate);
    parent.addChild(cubeBG);
    /**********************************************************
     * Fin creation du cube mobile selectionnable a la souris *
     **********************************************************/

    return parent;
  }

  /**
   * Etape 9 :
   * Methode main() nous permettant d'utiliser cette classe comme une applet
   * ou une application.
   * @param args arguments de la ligne de commande
   */
  public static void main(String[] args) {
    Frame frame = new MainFrame(new Collision(), 256, 256);
  }
}
