import java.awt.*;
import java.util.*;

import javax.media.j3d.*;
import javax.vecmath.*;

/**
 * Comportement personnalise sur les collisions
 */
public class DetecteurCollision extends Behavior {

  // Objet qui subit la collision
  protected Node node;

  // Apparence de l'objet node
  protected Appearance appearance;

  // Liste des criteres qui declenchent le stimulus pour le comportement
  // DetecteurCollision
  protected WakeupCriterion[] criteres;

  // Sert a declencher le stimulus si un des criteres de la liste ci-dessus
  // est verifie
  protected WakeupOr oredCriteria;

  // Definition des materiaux qui donnent les couleurs bleu, blanc et
  // rouge aux spheres
  Material bleu = new Material();
  Material blanc = new Material();
  Material rouge = new Material();

  /**
   * Constructeur
   * @param node objet qui va subir la collision
   * @param appearance apparence de l'objet node
   * @param bounds zone d'influence du comportement collision
   */
  public DetecteurCollision(Node node,
                            Appearance appearance,
                            Bounds bounds) {
    this.node = node;
    this.appearance = appearance;
    this.setSchedulingBounds(bounds);

    // Initialisation des materiaux
    bleu.setEmissiveColor(new Color3f(Color.blue));
    blanc.setEmissiveColor(new Color3f(Color.white));
    rouge.setEmissiveColor(new Color3f(Color.red));
  }

  /**
   * Initialise le comportement :
   * Le stimulus est declenche si un objet entre dans la zone de collision
   * de this.node, si il bouge dans la zone de collision ou si il sort de
   * la zone de collision.
   */
  public void initialize() {
    criteres = new WakeupCriterion[2];

    // Premier critere : entree dans la zone de collision
    WakeupOnCollisionEntry entreZoneCollision =
        new WakeupOnCollisionEntry(node);

    // Deuxieme critere : sortie de la zone de collision
    WakeupOnCollisionExit sortZoneCollision =
        new WakeupOnCollisionExit(node);

    criteres[0] = entreZoneCollision;
    criteres[1] = sortZoneCollision;

    // Stimulus declenche si l'un des 2 criteres est verifie
    oredCriteria = new WakeupOr(criteres);
    wakeupOn(oredCriteria);
  }

  /**
   * Le stimulus est traite ici.
   * @param criteria criteres ayant declenche le stimulus
   */
  public void processStimulus(Enumeration criteria) {

    // On boucle sur les criteres ayant declenche le comportement
    while (criteria.hasMoreElements()) {

      // On recupere le premier critere de l'enumeration
      WakeupCriterion theCriterion = (WakeupCriterion)criteria.nextElement();

      // Cas ou le critere correspond a l'entree dans la zone de collision
      if (theCriterion instanceof WakeupOnCollisionEntry) {
        if (node.getUserData().toString().equals("Sphere gauche")) {
          appearance.setMaterial(rouge);
        }
        else if (node.getUserData().toString().equals("Sphere droite")) {
          appearance.setMaterial(bleu);
        }
      }

      // Cas ou le critere correspond a la sortie de la zone de collision
      else if (theCriterion instanceof WakeupOnCollisionExit) {
        appearance.setMaterial(blanc);
      }
    }  // fin while (criteria.hasMoreElements())

    // Une fois le stimulus traite, on reinitialise le comportement
    wakeupOn(oredCriteria);
  }
}
